/*++

Copyright (c) 2007, Intel Corporation                                                         
All rights reserved. This program and the accompanying materials                          
are licensed and made available under the terms and conditions of the BSD License         
which accompanies this distribution.  The full text of the license may be found at        
http://opensource.org/licenses/bsd-license.php                                            
                                                                                          
THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,                     
WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.             

Module Name:

    Example.c
    
Abstract: 
    

Revision History
--*/

#include "Tiano.h"
#include "EfiDriverLib.h"
#include "EfiPrintLib.h"

#include EFI_PROTOCOL_DEFINITION (BlockIo)
#include EFI_PROTOCOL_DEFINITION (DiskIo)
#include EFI_PROTOCOL_DEFINITION (LoadedImage)

//
// Data Structures
//
typedef
VOID
(EFIAPI *MY_FUNC) (
  VOID   *Buffer,
  UINTN  Length
  );

typedef struct {
  UINTN    Index;
  MY_FUNC  MyFunc;
} MY_STRUCT;

typedef struct {
  EFI_STATUS  Status;
  UINTN       Index;
  VOID        **Inteface;
} MY_STRUCT_WITH_STATUS;

//
// Function Prototypes
EFI_STATUS
EFIAPI
MyFunc (
  VOID   *Buffer,
  UINTN  Length
  );

//
// Pre-Initialized Data Structures
//
MY_STRUCT MyStruct = {
  0, 
  &MyFunc 
};

EFI_BLOCK_IO_PROTOCOL  *MyBlockIoInstance;
EFI_DISK_IO_PROTOCOL   *MyDiskIoInstance;

MY_STRUCT_WITH_STATUS MyStructWithStatus[] = {
  {EFI_INVALID_PARAMETER, 0, &MyBlockIoInstance },
  {EFI_NOT_FOUND,         0, &MyDiskIoInstance  },
  {0,                     0, NULL               }
};

//
// Functions
//

UINTN
EFIAPI
EfiMain (
  UINTN  Operand
  )
{
  return Operand + 1;
}

UINTN
EFIAPI
EfiStart (
  UINTN  Operand
  )
{
  return Operand  * Operand;
}

EFI_STATUS
EFIAPI
MyFunc (
  VOID   *Buffer,
  UINTN  Length
  )
{
  return EFI_SUCCESS;
}


EFI_DRIVER_ENTRY_POINT (ExampleEntryPoint)

EFI_STATUS
EFIAPI
ExampleEntryPoint (
  IN EFI_HANDLE         ImageHandle,
  IN EFI_SYSTEM_TABLE   *SystemTable
  )
{
  UINTN                      Value;
  EFI_STATUS                 Status;
  EFI_LOADED_IMAGE_PROTOCOL  *LoadedImage;

  EfiInitializeDriverLib (ImageHandle, SystemTable);

  Print(L"PortDemo1\n\r");

  Value = 5;
  Print(L"Value = %d  f1(Value) = %d  f2(Value) = %d\n\r", Value, EfiMain(Value), EfiStart(Value));

  Status = gBS->HandleProtocol (
                  ImageHandle,
                  &gEfiLoadedImageProtocolGuid,
                  &LoadedImage
                  );
  switch (Status) {
  case EFI_SUCCESS:
    Print (L"Loaded Image Protocol found\n\r");
    break;
  case EFI_INVALID_PARAMETER:
    Print (L"Invalid parameter\n\r");
    break;
  default:
    Print (L"Unknown error\n\r");
    break;
  }

  Print(L"Done\n\r");

  return EFI_SUCCESS;
}